"""
    ZSS Attribute
    === =========

This object represents a CSS Attribute in a nice, editable, ZOPE object.
"""
# $Id: ZSSAttribute.py,v 1.2 2002/04/03 18:42:11 adrian Exp $
#
# $Log: ZSSAttribute.py,v $
# Revision 1.2  2002/04/03 18:42:11  adrian
# Fixed a "ZODB transaction leak" that was causing reading from a stylesheet to add transactions to data.fs. This would have slowed down access, especially over ZEO.
#
# Revision 1.1.1.1  2002/02/03 17:07:16  root
#
#
# Revision 1.3  2002/01/24 00:27:15  adrian
# Fixed minor bug that allowed the browser lists to get out of sync with the
# setting of the useBrowserVersions attribute.
# Removed useBrowserVersions attribute from ZSSAttribute
#
# Revision 1.2  2001/10/21 14:48:33  adrian
# no message
#
#
__version__ = '$Revision: 1.2 $'[11:-2]

from Globals import HTMLFile, MessageDialog
from OFS import SimpleItem
from OFS import PropertyManager
from DocumentTemplate import DT_HTML
import Acquisition
import urllib

import string

import BrowserType
 
manage_addZSSAttrForm=HTMLFile('www/manage_addZSSAttribute', globals())

def manage_addZSSAttr(self, id, value='', Browsers=[], REQUEST=None, submit=None):
    """ Add a ZSSAttr object """
    obj = ZSSAttr(str(id), str(value), Browsers)
    self._setObject(obj.getId(), obj)
    obj.useBrowserVersions = self.useBrowserVersions
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u, urllib.quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_workspace')
    return ''

class ZSSAttr(	PropertyManager.PropertyManager,
                SimpleItem.SimpleItem,
                BrowserType.BrowserType):
    """ Representation of Cascading Style Sheet Attributes """
    meta_type    = 'ZSS Attribute'
    id           = 'ZSSAttr'
    icon         = 'misc_/ZStyleSheet/ZSSAttr'
    manage_options    = (
        PropertyManager.PropertyManager.manage_options
        +(
            {'label': 'View', 'action': 'preview',
             'help': ('ZStyleSheet', 'Preview.stx')},
            {'label': 'Dump', 'action': 'dump',
             'help': ('ZStyleSheet', 'Dump.stx')},
        )
        +SimpleItem.SimpleItem.manage_options
    )
    __ac_permissions__= (
        PropertyManager.PropertyManager.__ac_permissions__
        +SimpleItem.SimpleItem.__ac_permissions__
    )
    _properties=(
        {'id' : 'value', 'type': 'string', 'mode': 'w'},
        {'id' : 'Browsers', 'type' : 'multiple selection', 'mode': 'w', 'select_variable': 'BrowserList'},
    )

    def __init__(self, id, value='', Browsers=[]):
        """ Initialise a ZSSAttr """
        self.id=id
        self.value=value
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers

    def _dump(self, parentContext=[], ignoreBrowsers=1, call=''):
        """ Add this attribute to a dump listing """
        return "/* Browsers=%s */\n\t%s: %s;" % (self.Browsers, self.getId(), DT_HTML.HTML(self.value, self.REQUEST, 'ZSSAttribute - _dump- %s' % self.getId()))

    def _value(self, parentContext=[], ignoreBrowsers=0, call=''):
        """ Return the un-rendered DTML for this item """
        # Fix up older objects by removing invalid attribute
#        try:
#        if hasattr(Acquisiton.aq_base(self).aq_base, 'useBrowserVersions'):
#            del self.useBrowserVersions
#            print "Deleting attribute from ZSSA %s" % (self.absolute_url(), )
#        except:
#            pass
        if self._testBrowser(self.Browsers) or ignoreBrowsers:
            return "<dtml-try>%s: %s;<dtml-except>/* ** WARNING ** Evaluation Failed - %s: %s*/</dtml-try>" % (self.getId(), self.value, self.getId(), DT_HTML.HTML(self.value, self.REQUEST))
        else:
            return "/* %s - Excluded for this browser */" % self.getId()

    def dump(self):
        """ Dump this attribute, showing the browser restrictions in comments. """
        dtml = '<dtml-var manage_page_header><dtml-var manage_tabs>'
        dtml = dtml + "<h1 align=center>Zope Style Sheet Attribute Dump</h1>"
        dtml = dtml + "<h2 align=center>%s</h2>" % self.getId()
        dtml = dtml + "<blockquote><pre>%s</pre></blockquote>" % self._dump()
        dtml = dtml + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        dtml = dtml + "<dtml-var manage_page_footer>"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZSSAttribute - dump - %s' % self.getId())
        return obj(self.aq_inner, self.REQUEST)

    def preview(self):
        """ Render a human viewable version of this ZSSAttribute """
        dtml = "<dtml-var manage_page_header><dtml-var manage_tabs>\n"
        dtml = dtml + "<h1 align=center>Zope Style Sheet Attribute Preview</h1>"
        dtml = dtml + "<h2 align=center>%s</h2>" % self.title_and_id()
        dtml = dtml + "<blockquote><pre>%s</pre></blockquote>" % self._value()
        dtml = dtml + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        dtml = dtml + "<dtml-var manage_page_footer>"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZSSAttribute - preview - %s' % self.title_and_id())
        return obj(self.aq_inner, self.REQUEST)

    def tag(self):
        """ Render this attribute """
        dtml = 'style="%s"' % self._value()
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZSSAttribute - %s' % self.getId())
        return obj(self.aq_inner, self.REQUEST)

    index_html = style = __call__ = __str__ = tag
