"""
    Zope Style Sheets
    ==== ===== ======

The aims of this product are:

* Represent Cascading Style Sheets in a ZOPE Object Tree
* Enable editing of CSS elements easily within ZOPE
* Enable previewing of the style sheet, and dump ZStyleSheets to xxx.css files
* Enable loading and converting xxx.css 'Flat Style Sheets' into ZStyleSheets
* Enable the downloading and converting of text/css objects from other web sites
"""
# $Id: ZStyleSheet.py,v 1.2 2003/05/16 22:39:16 adrian Exp $
#
# $Log: ZStyleSheet.py,v $
# Revision 1.2  2003/05/16 22:39:16  adrian
# Change to default to no-browser-versions, otherwise imports and text
# conversions barf.
#
# Revision 1.1.1.1  2002/02/03 17:07:16  root
#
#
# Revision 1.3  2002/01/24 00:27:15  adrian
# Fixed minor bug that allowed the browser lists to get out of sync with the
# setting of the useBrowserVersions attribute.
# Removed useBrowserVersions attribute from ZSSAttribute
#
# Revision 1.2  2001/10/21 14:48:33  adrian
# no message
#
#
__version__ = '$Revision: 1.2 $'[11:-2]

from Globals import HTML, HTMLFile, MessageDialog
from OFS import SimpleItem 
from OFS import ObjectManager
from OFS import PropertyManager
from App import Management
from DocumentTemplate import DT_HTML
import Acquisition
from AccessControl.Role import RoleManager
import urllib

import re
import string

import internals
import BrowserType
import ZSSRule
import ZSSConstant
import ZSSAttribute

StringType=type('')

manage_addZStyleSheetForm=HTMLFile('www/manage_addZStyleSheet', globals())

def manage_addZStyleSheet(self, id, title='', file='', Browsers=[], REQUEST=None, submit=None):
    """ Add a ZStyleSheet object """
    (id, title)=internals.cookId(str(id), str(title), str(file))
    obj = ZStyleSheet(id, title, Browsers)
    self._setObject(obj.getId(), obj)
    if (file):
        obj.manage_importCSS(file=file, Browsers=Browsers)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u, urllib.quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_workspace')
    return ''

class ZStyleSheet(  ObjectManager.ObjectManager,
                    PropertyManager.PropertyManager,
                    RoleManager,
                    SimpleItem.SimpleItem,
                    BrowserType.BrowserType):
    """ Representation of Cascading Style Sheets. """
    manage_addZSSRuleForm=ZSSRule.manage_addZSSRuleForm
    manage_addZSSRule=ZSSRule.manage_addZSSRule
    manage_addZSSConstantForm=ZSSConstant.manage_addZSSConstantForm
    manage_addZSSConstant=ZSSConstant.manage_addZSSConstant
    manage_importCSSForm=HTMLFile('www/manage_importCSS', globals())
    manage_ChangeBrowserForm=HTMLFile('www/manage_ChangeBrowser', globals())

    meta_type       = 'Z Style Sheet'
    id              = 'ZStyleSheet'
    icon            = 'misc_/ZStyleSheet/ZStyleSheet'
    all_meta_types = meta_types = (
        {'name': 'ZSS Rule', 'action': 'manage_addZSSRuleForm'},
        {'name': 'ZSS Constant', 'action': 'manage_addZSSConstantForm'},
    )
    manage_options  = (
        ObjectManager.ObjectManager.manage_options
        +PropertyManager.PropertyManager.manage_options
        +(
            {'label': 'View', 'action': 'preview',
             'help': ('ZStyleSheet', 'Preview.stx')},
            {'label': 'Dump', 'action': 'dump',
             'help': ('ZStyleSheet', 'Dump.stx')},
            {'label': 'Change Browser', 'action': 'manage_ChangeBrowserForm',
             'help': ('ZStyleSheet', 'ChangeBrowser.stx')},
            {'label': 'Import CSS', 'action': 'manage_importCSSForm',
             'help': ('ZStyleSheet', 'ImportCSS.stx')},
        )
        +SimpleItem.SimpleItem.manage_options
        )
    __ac_permissions__= (
        PropertyManager.PropertyManager.__ac_permissions__
        +SimpleItem.SimpleItem.__ac_permissions__
     )
    _properties	=   (
        {'id': 'title', 'type': 'string', 'mode': 'w'},
        {'id': 'Browsers', 'type': 'multiple selection', 'mode': 'w', 'select_variable': 'BrowserList'},
        {'id': 'linkstyle', 'type': 'selection', 'mode': 'w', 'select_variable': 'linkStyleList'},
        {'id': 'useBrowserVersions', 'type': 'boolean', 'mode': 'w'},
    )
    linkStyleList = (
        'Normal Z Style Sheet',
        'Insert a Style Block',
        'ZMI Compatible',
    )

    def __init__(self, id, title='', Browsers=[]):
        """ Initialise a ZStyleSheet """
        self.id=id
        self.title=title
        self.useBrowserVersions = 0
        if (not(Browsers)):
            Browsers = self.BrowserList()
        self.Browsers = Browsers
        self.linkstyle = self.linkStyleList[0]

    def manage_importCSS(self, file='', Browsers=[], REQUEST=None):
        """ Load, interpret and import a CSS file """
        n=1<<16
        if type(file) is StringType:
            size=len(file)
            if size < n:
                data=file
            else:
                data=Pdata(file)
        else:
            seek=file.seek
            read=file.read
            seek(0,2)
            size=end=file.tell()
            if size <= 2*n:
                seek(0)
                if size < n:
                    data=read(size)
                else:
                    data=Pdata(read(size))

        if (not(Browsers)):
            Browsers=self.BrowserList()

        if not(data):
            return MessageDialog(
                title  ='Failed!',
                message='Your static stylesheet was empty!!',
                action ='manage_main')
        else:
            self._WalkCSS(data, Browsers)

        if REQUEST:
            return MessageDialog(
                title  ='Success!',
                message='Your static stylesheet has been added',
                action ='manage_main')

    def _updateProperty(self, id, value):
        """ Catch changes to properties """
        oldUBV = self.useBrowserVersions
        PropertyManager.PropertyManager._updateProperty(self, id, value)
        if id == 'useBrowserVersions':
            if oldUBV!=value:
                self.manage_ChangeBrowser()

    def manage_ChangeBrowser(self, Browsers=[], REQUEST=None):
        """ Walk the tree, changing the Browsers entry at each node """
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers
        objs=self.objectValues('ZSS Rule')
        for i in objs:
            i.useBrowserVersions=self.useBrowserVersions
            i.manage_ChangeBrowser(Browsers)
        if REQUEST:
            return MessageDialog(
                title  ='Success!',
                message='The browser settings have been changed',
                action ='manage_main')

    def _WalkCSS(self, CSS, Browsers):
        """ Walk a CSS file, ignoring comments, extracting rules, selectors and attributes """
        CSS=re.sub("\n|\r|\t", " ", CSS)
        CSS=re.sub("/\*.*?\*/", " ", CSS)
        CSS=re.sub("}", "\n", CSS)
        for Rule in re.split("\n", CSS):
            try:
                (Selectors, Attributes)=re.split("{", Rule)
            except:
                pass
            else:
# Split the rules, merge and munge
                SelList=[]
                for Selector in re.split(",", Selectors):
                    SelList.append(internals._Rule(Selector, Attributes, SelList))
# Create the tree
                for S in SelList:
                    S._create(self, Browsers)

    def _dump(self, ignoreBrowsers=1, call='_dump'):
        """ Add this stylesheet to a dump listing """
        return "/* Browsers=%s */\n%s" % (self.Browsers, self._value(ignoreBrowsers=ignoreBrowsers, call=call),)

    def _children(self, TYPE='', HEAD='', TAIL='', ignoreBrowsers=0, call='_value'):
        """ This does the children of this item """
        strret = ""
        objs = self.objectValues(TYPE)
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            Icall = getattr(i, call)
            strret = strret + "%s%s%s" % (HEAD, Icall(ignoreBrowsers=ignoreBrowsers, call=call), TAIL)
        return strret

    def _value(self, ignoreBrowsers=0, call='_value'):
        """ Render the stylesheet from here down """
        strret=""
        if self._testBrowser(self.Browsers) or ignoreBrowsers:
            strret = strret + self._children(TYPE='ZSS Rule', HEAD='', TAIL='\n', ignoreBrowsers=ignoreBrowsers, call=call)
        else:
            strret = "/* %s - Excluded for this browser */" % self.id
        strret = strret + "/* End of sheet */\n"
        return strret

    def dump(self):
        """ Dump this stylesheet, showing the browser restrictions in comments. """
        if not hasattr(self, 'linkstyle'):
            self.linkstyle = self.linkStyleList[0]
        dtml = '<dtml-var manage_page_header><dtml-var manage_tabs>'
        dtml = dtml + "<h1 align=center>Zope Style Sheet Dump</h1>"
        dtml = dtml + "<h2 align=center>%s</h2>" % self.title_and_id()
        dtml = dtml + "<blockquote><pre>%s</pre></blockquote>" % self._dump()
        dtml = dtml + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        dtml = dtml + "<dtml-var manage_page_footer>"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZStyleSheet - dump - %s' % self.getId())
        return obj(self.aq_inner, self.REQUEST)

    def preview(self):
        """ Render a human viewable version of this Z Style Sheet """
        if not hasattr(self, 'linkstyle'):
            self.linkstyle = self.linkStyleList[0]
        dtml = "<dtml-var manage_page_header><dtml-var manage_tabs>\n"
        dtml = dtml + "<h1 align=center>Zope Style Sheet Preview</h1>"
        dtml = dtml + "<h2 align=center>%s</h2>" % self.title_and_id()
        dtml = dtml + "<blockquote><pre>%s</pre></blockquote>" % self._value()
        dtml = dtml + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        dtml = dtml + "<dtml-var manage_page_footer>"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZStyleSheet - preview - %s' % self.title_and_id())
        return obj(self.aq_inner, self.REQUEST)

    def tag(self):
        """ Add a LINK tag to the HTML page """
        return '<link rel="stylesheet" type="text/css" href="%s" />' % (self.absolute_url())

    def style(self):
        """ Render the stylesheet as a <style> </style> section """
        if self._testBrowser(self.Browsers):
            obj = DT_HTML.HTML('<style type="text/css"><!--\n%s\n--></style>' % (self._value()),
                            self.REQUEST, 'ZStyleSheet - style - %s' % self.title_and_id())
            return obj(self.aq_inner, self.REQUEST)
        return ""

    def __str__(self):
        """ Choose which function to call based on self.linkstyle """
        retstr = ""
        if not hasattr(self, 'linkstyle'):
            self.linkstyle = self.linkStyleList[0]
        #   'Traditional'   -   Insert a link tag
        if self.linkstyle == self.linkStyleList[0]:
            retstr = self.tag()
        #   'Style'         -   Insert a style block
        elif self.linkstyle == self.linkStyleList[1]:
            retstr = self.style()
        #   'ZMI'           -   Return just the style content
        elif self.linkstyle == self.linkStyleList[2]:
            obj = DT_HTML.HTML(self._value(), self.REQUEST, 'ZStyleSheet - __str__ - %s' % self.title_and_id())
            retstr = obj(self.aq_inner, self.REQUEST)
        return retstr

    __call__ = __str__

    def index_html(self):
        """ Render the stylesheet as text/css """
        if not hasattr(self, 'linkstyle'):
            self.linkstyle = self.linkStyleList[0]
        dtml = """<dtml-call "RESPONSE.setHeader('Content-Type', 'text/css')">"""
        dtml = dtml + "/* Cascading Style Sheet created using ZStyleSheet */\n"
        dtml = dtml + self._value()
        dtml = dtml + "/* Written by Haqa http://www.zope.org/Members/haqa */\n"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZStyleSheet - index_html - %s' % self.title_and_id())
        return obj(self.aq_inner, self.REQUEST)
